import setuptools

setuptools.setup(
    name="pyvter",
    version="0.0.3",
    author="jhd",
    author_email="author@example.com",
    description="Interactive Statistics Workflows",
    long_description="Interactive Statistics Workflows",
    long_description_content_type="text/markdown",
    url="https://github.com/jhd/pyvter",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU Affero General Public License v3",
        "Operating System :: OS Independent",
    ],
)
