
from _anova import Anova
from _anova1way import Anova1way
from _chisquare1way import ChiSquare1way
from _chisquare2way import ChiSquare2way
from _correlation import Correlation
from _descriptives import Descriptives
from _histogram import Histogram
from _marginals import Marginals
from _ttest import Ttest
from _stats import *
from _pstat import *
from qsturng import *
