
from _box_plot import box_plot
from _histogram_plot import histogram_plot
from _interaction_plot import interaction_plot
from _scatter_matrix import scatter_matrix
from _scatter_plot import scatter_plot
from support import *
